require 'test_helper'



class GoogleDocTest < ActiveSupport::TestCase
  attr_reader :google_doc, :install_process
  def setup
    GoogleDoc.mock_test!
    @google_doc = GoogleDoc.new("installer_cached", nil)
    @install_process = google_doc.use_cases.first
  end
  
  def self.skip_test name, &block
    puts "WARNING: skipping test #{name}"
  end
  def pretty_steps(steps)
    "What we got: \n#{steps.map(&:short_description).join("\n")}"
  end
  
  test "basic stuff" do
    assert_match "<GoogleDoc",google_doc.inspect
    assert_equal 5, google_doc.use_cases.length, "Should be right number of use cases"

  end
  
  test "complex doc" do
    @file_sync_doc = GoogleDoc.new("file_sync_cached",nil)
    
    assert_not_nil @file_sync_doc.introduction, "File sync should have introduction"
  end
  
  test "test models are basically parsed out correctly" do
    assert_equal "3.1    Installer computer setup process", install_process.title
    assert_equal 5, install_process.journeys.length, "Should be right number of variations"
    assert_match "User selects their location", install_process.find_step('9').title_without_number, "Should be able to search for a step and get the right title"
    assert_match "User types in their password correctly from the list and clicks Next.", install_process.find_step('8.a.4').title_without_number
    
    assert_match "The Ubuntu Live CD is the first experience many new users will have of Ubuntu", @google_doc.introduction, "Should include the intro"
    #assert_no_match /Interaction architecture/, @google_doc.introduction, "Shouldn't include stuff before or after the intro"
    
  end

  test "Completion and bug-related tests" do
    step_with_bug = install_process.find_step('13').bug
    assert_not_nil step_with_bug, "Should be a bug associated with this use case step"
    assert !(step_with_bug =~ /bug\#568979/), "Should strip the bug number from the title"
   
   
    assert_nil install_process.find_step(1).marked_done?
    assert install_process.find_step(3).marked_done?
    
    use_case = google_doc.use_cases.find{ |u| u.number== '2.1.A'}
    
    dodgy_bug_step = use_case.find_step('1.b.1')
    assert_not_equal '', dodgy_bug_step.title, "Should not lose the title when finding an attached bug"
    assert_no_match /598095/, dodgy_bug_step.title, "Shouldn't just leave the bug number!"
    assert_not_nil dodgy_bug_step.bug, "Should find the bug number"
  end

  test "Use case step sorting" do
    sorted = install_process.all_steps.sort
    assert_equal 17, sorted.find_index( install_process.find_step('8') ),"Primary scenario steps should come after use case steps. #{pretty_steps(sorted)}"
  end
  
  test "John-style map-related tests" do
    use_case = google_doc.use_cases.find{ |u| u.number== '2.1.A'}
    assert_not_nil use_case
    assert_equal 9, use_case.journeys.first.steps.length, "Should be right number of steps in primary scenario"
    assert_equal 10, use_case.journeys.length, "Should find right number of journeys"
    
    variation_of_first_step = use_case.find_journey_by_id("1_a")
    assert_equal 4, variation_of_first_step.steps.length, "Should find right number of steps for a variation of step 1 (#{ pretty_steps variation_of_first_step.steps})"
    
    assert_equal 3, variation_of_first_step.connections.length, "Should be right number of connections (\n#{variation_of_first_step.connections.map(&:short_description).join("\n")})"
    
    
    journey_3_a = use_case.find_journey_by_id("3_a")
    assert_equal 12, journey_3_a.steps.length, "Should have enough steps in this journey #{ pretty_steps( journey_3_a.steps ) })"
    
  end

  test "Parse error handling" do
    use_case = google_doc.use_cases.find{ |u| u.number == '2.17' }
    bad_step = use_case.find_step('3.b.1')
    assert_match /ERROR/, bad_step.title, "Should be an error shown when return-to step is not found"
  end
  

  test "Map-related tests" do
  
    journey_6_a = install_process.find_journey_by_id("6_a")
    
    step_6  = install_process.find_step('6')
    assert_not_nil journey_6_a
    assert_not_nil step_6
    
    assert_equal "6.a.1", install_process.find_step("6_a").number, "When requesting a journey title step, should retrieve the first step in that journey"
    
    assert journey_6_a.steps.include?(step_6), "Should find step 6 in journey 6.a (RETURN TO BRANCHING STEP)"
    assert_equal 8, journey_6_a.steps.find_index(step_6), "When returning to branching step, should be directly after variation steps"
    
    journey_8_a = install_process.find_journey_by_id("8_a")
    assert_equal 9, journey_8_a.steps.find_index(install_process.find_step('8.a.1')), "Variation steps should be in correct order"
   
    assert_equal 18, journey_6_a.steps.length, "Should still be enough steps in the journey"
    
    step_8 = install_process.find_step('8')
    assert_nil journey_8_a.steps.find_index(step_8), "Variation should not include skipped step. #{ pretty_steps journey_8_a.steps} "
    
    bad_connection = journey_6_a.connections.find{ |c| c.from == install_process.find_step('5') and c.to == install_process.find_step('6')  }
    
    assert_nil bad_connection, "Should not be a connection from a branched primary scenario step to the returned-to step"
    
    install_process.journeys.each do |journey|
      assert !journey.connections.empty?, "Journey connections must be present (Not found in #{journey.title}; #{pretty_steps journey.steps}; connections:#{journey.connections})"
    end
    
    #assert_equal 10,journey_6_a.connections.length, "Shouldn't lose all the connections"
    
  end



end
